@@ -31,11 +31,12 @@ import hashlib
 import hmac
 from typing import Union, Mapping, Optional
 
+import electrum_ecc as ecc
+
 from .util import assert_bytes, InvalidPassword, to_bytes, to_string, WalletFileException, versiontuple
 from .i18n import _
 from .logging import get_logger
 
-
 _logger = get_logger(__name__)
 
 
@@ -348,12 +349,9 @@ def ripemd(x: bytes) -> bytes:
         md = ripemd.new(x)
         return md.digest()
 
+
 def hmac_oneshot(key: bytes, msg: bytes, digest) -> bytes:
-    if hasattr(hmac, 'digest'):
-        # requires python 3.7+; faster
-        return hmac.digest(key, msg, digest)
-    else:
-        return hmac.new(key, msg, digest).digest()
+    return hmac.digest(key, msg, digest)
 
 
 def chacha20_poly1305_encrypt(
@@ -410,6 +408,9 @@ def chacha20_poly1305_decrypt(
 
 
 def chacha20_encrypt(*, key: bytes, nonce: bytes, data: bytes) -> bytes:
+    """note: for any new protocol you design, please consider using chacha20_poly1305_encrypt instead
+             (for its Authenticated Encryption property).
+    """
     assert isinstance(key, (bytes, bytearray))
     assert isinstance(nonce, (bytes, bytearray))
     assert isinstance(data, (bytes, bytearray))
@@ -443,3 +444,59 @@ def chacha20_decrypt(*, key: bytes, nonce: bytes, data: bytes) -> bytes:
         decryptor = cipher.decryptor()
         return decryptor.update(data)
     raise Exception("no chacha20 backend found")
+
+
+def ecies_encrypt_message(
+    ec_pubkey: 'ecc.ECPubkey',
+    message: bytes,
+    *,
+    magic: bytes = b'BIE1',
+) -> bytes:
+    """
+        ECIES encryption/decryption methods; AES-128-CBC with PKCS7 is used as the cipher; hmac-sha256 is used as the mac
+    """
+    assert_bytes(message)
+    ephemeral = ecc.ECPrivkey.generate_random_key()
+    ecdh_key = (ec_pubkey * ephemeral.secret_scalar).get_public_key_bytes(compressed=True)
+    key = hashlib.sha512(ecdh_key).digest()
+    iv, key_e, key_m = key[0:16], key[16:32], key[32:]
+    ciphertext = aes_encrypt_with_iv(key_e, iv, message)
+    ephemeral_pubkey = ephemeral.get_public_key_bytes(compressed=True)
+    encrypted = magic + ephemeral_pubkey + ciphertext
+    mac = hmac_oneshot(key_m, encrypted, hashlib.sha256)
+    return base64.b64encode(encrypted + mac)
+
+
+def ecies_decrypt_message(
+    ec_privkey: 'ecc.ECPrivkey',
+    encrypted: Union[str, bytes],
+    *,
+    magic: bytes = b'BIE1',
+) -> bytes:
+    encrypted = base64.b64decode(encrypted, validate=True)  # type: bytes
+    if len(encrypted) < 85:
+        raise Exception('invalid ciphertext: length')
+    magic_found = encrypted[:4]
+    ephemeral_pubkey_bytes = encrypted[4:37]
+    ciphertext = encrypted[37:-32]
+    mac = encrypted[-32:]
+    if magic_found != magic:
+        raise Exception('invalid ciphertext: invalid magic bytes')
+    try:
+        ephemeral_pubkey = ecc.ECPubkey(ephemeral_pubkey_bytes)
+    except ecc.InvalidECPointException as e:
+        raise Exception('invalid ciphertext: invalid ephemeral pubkey') from e
+    ecdh_key = (ephemeral_pubkey * ec_privkey.secret_scalar).get_public_key_bytes(compressed=True)
+    key = hashlib.sha512(ecdh_key).digest()
+    iv, key_e, key_m = key[0:16], key[16:32], key[32:]
+    if mac != hmac_oneshot(key_m, encrypted[:-32], hashlib.sha256):
+        raise InvalidPassword()
+    return aes_decrypt_with_iv(key_e, iv, ciphertext)
+
+
+def get_ecdh(priv: bytes, pub: bytes) -> bytes:
+    pt = ecc.ECPubkey(pub) * ecc.string_to_number(priv)
+    return sha256(pt.get_public_key_bytes())
+
+def privkey_to_pubkey(priv: bytes) -> bytes:
+    return ecc.ECPrivkey(priv[:32]).get_public_key_bytes()
