@@ -8,17 +8,18 @@ import time
 import csv
 import decimal
 from decimal import Decimal
-from typing import Sequence, Optional, Mapping, Dict, Union, Any, Tuple
+from typing import Sequence, Optional, Mapping, Dict, Union, Tuple
 
-from aiorpcx.curio import timeout_after, TaskTimeout, ignore_after
+from aiorpcx.curio import timeout_after, ignore_after
 import aiohttp
 
 from . import util
 from .bitcoin import COIN
 from .i18n import _
-from .util import (ThreadJob, make_dir, log_exceptions, OldTaskGroup,
-                   make_aiohttp_session, resource_path, EventListener, event_listener, to_decimal,
-                   timestamp_to_datetime)
+from .util import (
+    ThreadJob, make_dir, log_exceptions, OldTaskGroup, make_aiohttp_session, resource_path, EventListener,
+    event_listener, to_decimal, timestamp_to_datetime
+)
 from .util import NetworkRetryManager
 from .network import Network
 from .simple_config import SimpleConfig
@@ -45,7 +46,7 @@ class ExchangeBase(Logger):
 
     def __init__(self, on_quotes, on_history):
         Logger.__init__(self)
-        self._history = {}  # type: Dict[str, Dict[str, str]]
+        self._history = {}  # type: Dict[str, Dict[str, str | float]]
         self._quotes = {}  # type: Dict[str, Optional[Decimal]]
         self._quotes_timestamp = 0  # type: Union[int, float]
         self.on_quotes = on_quotes
@@ -93,14 +94,14 @@ class ExchangeBase(Logger):
             self.logger.exception(f"failed fx quotes: {repr(e)}")
             self.on_quotes()
         else:
-            self.logger.info("received fx quotes")
+            self.logger.debug("received fx quotes")
             self._quotes_timestamp = time.time()
             self.on_quotes(received_new_data=True)
 
     @staticmethod
     def _read_historical_rates_from_file(
         *, exchange_name: str, ccy: str, cache_dir: str,
-    ) -> Tuple[Optional[dict], Optional[float]]:
+    ) -> Tuple[Optional[Dict[str, str]], Optional[float]]:
         filename = os.path.join(cache_dir, f"{exchange_name}_{ccy}")
         if not os.path.exists(filename):
             return None, None
@@ -124,6 +125,7 @@ class ExchangeBase(Logger):
         )
         if not h:
             return None
+        assert timestamp is not None
         h['timestamp'] = timestamp
         self._history[ccy] = h
         self.on_history()
@@ -133,16 +135,22 @@ class ExchangeBase(Logger):
     def _write_historical_rates_to_file(
         *, exchange_name: str, ccy: str, cache_dir: str, history: Dict[str, str],
     ) -> None:
+        # sanity check types of history dict
+        assert 'timestamp' not in history
+        for key, rate in history.items():
+            assert isinstance(key, str), f"{exchange_name=}. {ccy=}. {key=!r}. {rate=!r}"
+            assert isinstance(rate, str), f"{exchange_name=}. {ccy=}. {key=!r}. {rate=!r}"
+        # write to file
         filename = os.path.join(cache_dir, f"{exchange_name}_{ccy}")
         with open(filename, 'w', encoding='utf-8') as f:
-            f.write(json.dumps(history))
+            f.write(json.dumps(history, sort_keys=True))
 
     @log_exceptions
     async def get_historical_rates_safe(self, ccy: str, cache_dir: str) -> None:
         try:
             self.logger.info(f"requesting fx history for {ccy}")
             h_new = await self.request_history(ccy)
-            self.logger.info(f"received fx history for {ccy}")
+            self.logger.debug(f"received fx history for {ccy}")
         except (aiohttp.ClientError, asyncio.TimeoutError, OSError) as e:
             self.logger.info(f"failed fx history: {repr(e)}")
             return
@@ -150,7 +158,7 @@ class ExchangeBase(Logger):
             self.logger.exception(f"failed fx history: {repr(e)}")
             return
         # cast rates to str
-        h_new = {date_str: str(rate) for (date_str, rate) in h_new.items()}
+        h_new = {date_str: str(rate) for (date_str, rate) in h_new.items()}  # type: Dict[str, str]
         # merge old history and new history. resolve duplicate dates using new data.
         h_old, _timestamp = self._read_historical_rates_from_file(
             exchange_name=self.name(), ccy=ccy, cache_dir=cache_dir,
@@ -161,7 +169,7 @@ class ExchangeBase(Logger):
         self._write_historical_rates_to_file(
             exchange_name=self.name(), ccy=ccy, cache_dir=cache_dir, history=h,
         )
-        h['timestamp'] = time.time()
+        h['timestamp'] = time.time()  # note: this is the only item in h that has a float value
         self._history[ccy] = h
         self.on_history()
 
@@ -208,6 +216,7 @@ class ExchangeBase(Logger):
             return Decimal('NaN')
         return Decimal(rate)
 
+
 class Yadio(ExchangeBase):
 
     async def get_currencies(self):
@@ -218,6 +227,7 @@ class Yadio(ExchangeBase):
         json = await self.get_json('api.yadio.io', '/rate/%s/BTC' % ccy)
         return {ccy: to_decimal(json['rate'])}
 
+
 class BitcoinAverage(ExchangeBase):
     # note: historical rates used to be freely available
     # but this is no longer the case. see #5188
@@ -247,9 +257,8 @@ class BitcoinVenezuela(ExchangeBase):
         return ['ARS', 'EUR', 'USD', 'VEF']
 
     async def request_history(self, ccy):
-        json = await self.get_json('api.bitcoinvenezuela.com',
-                             "/historical/index.php?coin=BTC")
-        return json[ccy +'_BTC']
+        json = await self.get_json('api.bitcoinvenezuela.com', "/historical/index.php?coin=BTC")
+        return json[ccy + '_BTC']
 
 
 class Bitbank(ExchangeBase):
@@ -259,6 +268,35 @@ class Bitbank(ExchangeBase):
         return {'JPY': to_decimal(json['data']['last'])}
 
 
+class BitFinex(ExchangeBase):
+
+    async def get_currencies(self):
+        json = await self.get_json(
+            'api-pub.bitfinex.com',
+            f"/v2/conf/pub:list:pair:exchange")
+        pairs = [pair for pair in json[0]
+                 if len(pair) == 6 and pair[:3] == "BTC"]
+        return [pair[3:] for pair in pairs]
+
+    def history_ccys(self):
+        return CURRENCIES[self.name()]
+
+    async def get_rates(self, ccy):
+        # ref https://docs.bitfinex.com/reference/rest-public-ticker
+        json = await self.get_json(
+            'api-pub.bitfinex.com',
+            f"/v2/ticker/tBTC{ccy}")
+        return {ccy: to_decimal(json[6])}
+
+    async def request_history(self, ccy):
+        # ref https://docs.bitfinex.com/reference/rest-public-candles
+        history = await self.get_json(
+            'api.bitfinex.com',
+            f"/v2/candles/trade:1D:tBTC{ccy}/hist?limit=10000")
+        return dict([(timestamp_to_datetime(h[0] // 1000, utc=True).strftime('%Y-%m-%d'), str(h[2]))
+                     for h in history])
+
+
 class BitFlyer(ExchangeBase):
 
     async def get_rates(self, ccy):
@@ -283,14 +321,50 @@ class Bitso(ExchangeBase):
 class BitStamp(ExchangeBase):
 
     async def get_currencies(self):
-        return ['USD', 'EUR']
+        # ref https://www.bitstamp.net/api/#tag/Tickers/operation/GetCurrencyPairTickers
+        json = await self.get_json(
+            'www.bitstamp.net',
+            f"/api/v2/ticker/")
+        pairs = [ticker["pair"] for ticker in json]
+        pairs = [pair for pair in pairs
+                 if len(pair) == 7 and pair[:4] == "BTC/"]
+        return [pair[4:] for pair in pairs]
 
     async def get_rates(self, ccy):
+        # ref https://www.bitstamp.net/api/#tag/Tickers/operation/GetMarketTicker
         if ccy in CURRENCIES[self.name()]:
             json = await self.get_json('www.bitstamp.net', f'/api/v2/ticker/btc{ccy.lower()}/')
             return {ccy: to_decimal(json['last'])}
         return {}
 
+    def history_ccys(self):
+        return CURRENCIES[self.name()]
+
+    async def request_history(self, ccy):
+        # ref https://www.bitstamp.net/api/#tag/Market-info/operation/GetOHLCData
+        merged_history = {}
+        history_starts = 1313625600  # for BTCUSD pair (probably earliest)
+        items_per_request = 1000
+        step = 86400
+
+        async def populate_history(endtime: int):
+            history = await self.get_json(
+                'www.bitstamp.net',
+                f"/api/v2/ohlc/btc{ccy.lower()}/?step={step}&limit={items_per_request}&end={endtime}")
+            history = dict([
+                (timestamp_to_datetime(int(h["timestamp"]), utc=True).strftime('%Y-%m-%d'), str(h["close"]))
+                for h in history["data"]["ohlc"]])
+            merged_history.update(history)
+
+        async with OldTaskGroup() as group:
+            endtime = int(time.time())
+            while True:
+                if endtime < history_starts:
+                    break
+                await group.spawn(populate_history(endtime=endtime))
+                endtime = endtime - items_per_request * step
+        return merged_history
+
 
 class Bitvalor(ExchangeBase):
 
@@ -380,6 +454,7 @@ class CoinGecko(ExchangeBase):
         return CURRENCIES[self.name()]
 
     async def request_history(self, ccy):
+        # ref https://docs.coingecko.com/v3.0.1/reference/coins-id-market-chart
         num_days = 365
         # Setting `num_days = "max"` started erroring (around 2024-04) with:
         # > Your request exceeds the allowed time range. Public API users are limited to querying
@@ -429,6 +504,7 @@ class itBit(ExchangeBase):
 class Kraken(ExchangeBase):
 
     async def get_rates(self, ccy):
+        # ref https://docs.kraken.com/api/docs/rest-api/get-ticker-information
         ccys = ['EUR', 'USD', 'CAD', 'GBP', 'JPY']
         pairs = ['XBT%s' % c for c in ccys]
         json = await self.get_json('api.kraken.com',
@@ -436,6 +512,10 @@ class Kraken(ExchangeBase):
         return dict((k[-3:], to_decimal(v['c'][0]))
                      for k, v in json['result'].items())
 
+    # async def request_history(self, ccy):
+    #     # ref https://docs.kraken.com/api/docs/rest-api/get-ohlc-data
+    #     pass  # limited to last 720 steps (step can by 1 day / 7 days / 15 days)
+
 
 class MercadoBitcoin(ExchangeBase):
 
@@ -526,11 +606,14 @@ def get_exchanges_and_currencies():
                 for name, klass in exchanges.items():
                     exchange = klass(None, None)
                     await group.spawn(get_currencies_safe(name, exchange))
-    loop = util.get_asyncio_loop()
+
+    loop = asyncio.new_event_loop()
     try:
         loop.run_until_complete(query_all_exchanges_for_their_ccys_over_network())
     except Exception as e:
         pass
+    finally:
+        loop.close()
     with open(path, 'w', encoding='utf-8') as f:
         f.write(json.dumps(d, indent=4, sort_keys=True))
     return d
